'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.createClassName = createClassName;
exports.isMenuObject = isMenuObject;
exports.checkActive = checkActive;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Create class names by using tame name and classes array or menu
 *
 * @param {String} theme
 * @param {Array|String} classNames
 * @returns {string|*}
 */
function createClassName(_ref) {
  var theme = _ref.theme;
  var classNames = _ref.classNames;

  classNames = _lodash2["default"].isArray(classNames) ? classNames : [classNames];
  return classNames.filter(function (className) {
    return !!className;
  }).map(function (className) {
    return theme + '_' + className;
  }).join(' ');
}

/**
 * check if its a normal menu object by checking for path.
 * @param {Object} component
 * @returns {boolean|*}
 */
function isMenuObject(component) {
  return (typeof component === 'undefined' ? 'undefined' : _typeof(component)) === 'object' && component.path;
}

/**
 * Check if menu is active
 * @param {Object} menu
 * @param {Object} location
 */
function checkActive(_ref2) {
  var menu = _ref2.menu;
  var location = _ref2.location;

  switch (_typeof(menu.active)) {
    case 'boolean':
      return menu.active;
      break;
    case 'function':
      return menu.active();
      break;
    case 'undefined' || 'string':
      var path = menu.active || menu.path;
      return location.pathname == path;
      break;
  }
}